@@ -61,6 +61,7 @@ typedef enum {
 typedef struct {
         SoupClientMessageIO iface;
 
+        SoupConnection *conn;
         GIOStream *stream;
         GInputStream *istream;
         GOutputStream *ostream;
@@ -369,6 +370,11 @@ io_write_ready (GObject                  *stream,
 {
         GError *error = NULL;
 
+        if (io->error) {
+                g_clear_pointer (&io->write_source, g_source_unref);
+                return G_SOURCE_REMOVE;
+        }
+
         while (nghttp2_session_want_write (io->session) && !error)
                 io_write (io, FALSE, NULL, &error);
 
@@ -433,6 +439,17 @@ io_read_ready (GObject                  *stream,
         GError *error = NULL;
         gboolean progress = TRUE;
 
+        if (io->error) {
+                g_clear_pointer (&io->read_source, g_source_unref);
+                return G_SOURCE_REMOVE;
+        }
+
+        /* Mark the connection as in use to make sure it's not disconnected while
+         * processing pending messages, for example if a goaway is received.
+         */
+        if (io->conn)
+                soup_connection_set_in_use (io->conn, TRUE);
+
         while (nghttp2_session_want_read (io->session) && progress) {
                 progress = io_read (io, FALSE, NULL, &error);
                 if (progress) {
@@ -444,13 +461,19 @@ io_read_ready (GObject                  *stream,
 
         if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
                 g_error_free (error);
+                if (io->conn)
+                        soup_connection_set_in_use (io->conn, FALSE);
                 return G_SOURCE_CONTINUE;
         }
 
         if (error)
                 set_io_error (io, error);
 
+        io->is_shutdown = TRUE;
+
         g_clear_pointer (&io->read_source, g_source_unref);
+        if (io->conn)
+                soup_connection_set_in_use (io->conn, FALSE);
         return G_SOURCE_REMOVE;
 }
 
@@ -607,23 +630,35 @@ on_frame_recv_callback (nghttp2_session     *session,
                         gpointer             user_data)
 {
         SoupClientMessageIOHTTP2 *io = user_data;
-        SoupHTTP2MessageData *data = nghttp2_session_get_stream_user_data (session, frame->hd.stream_id);
+        SoupHTTP2MessageData *data;
 
-        h2_debug (io, data, "[RECV] [%s] Recieved (%u)", frame_type_to_string (frame->hd.type), frame->hd.flags);
+        if (frame->hd.stream_id == 0) {
+                h2_debug (io, NULL, "[RECV] [%s] Recieved (%u)", frame_type_to_string (frame->hd.type), frame->hd.flags);
+
+                switch (frame->hd.type) {
+                case NGHTTP2_GOAWAY:
+                        h2_debug (io, NULL, "[RECV] GOAWAY: error=%s, last_stream_id=%d %s",
+                                  nghttp2_http2_strerror (frame->goaway.error_code),
+                                  frame->goaway.last_stream_id,
+                                  frame->goaway.opaque_data ? (char *)frame->goaway.opaque_data : "");
+                        handle_goaway (io, frame->goaway.error_code, frame->goaway.last_stream_id);
+                        io->is_shutdown = TRUE;
+                        soup_client_message_io_http2_terminate_session (io);
+                        break;
+                case NGHTTP2_WINDOW_UPDATE:
+                        h2_debug (io, NULL, "[RECV] WINDOW_UPDATE: increment=%d, total=%d", frame->window_update.window_size_increment,
+                                  nghttp2_session_get_remote_window_size (session));
+                        break;
+                }
 
-        if (frame->hd.type == NGHTTP2_GOAWAY) {
-                h2_debug (io, data, "[RECV] GOAWAY: error=%s, last_stream_id=%d %s",
-                          nghttp2_http2_strerror (frame->goaway.error_code),
-                          frame->goaway.last_stream_id,
-                          frame->goaway.opaque_data ? (char *)frame->goaway.opaque_data : "");
-                handle_goaway (io, frame->goaway.error_code, frame->goaway.last_stream_id);
-                io->is_shutdown = TRUE;
-                soup_client_message_io_http2_terminate_session (io);
                 return 0;
         }
 
+        data = nghttp2_session_get_stream_user_data (session, frame->hd.stream_id);
+        h2_debug (io, data, "[RECV] [%s] Recieved (%u)", frame_type_to_string (frame->hd.type), frame->hd.flags);
+
         if (!data) {
-                if (frame->hd.stream_id != 0 && !(frame->hd.flags & NGHTTP2_FLAG_END_STREAM))
+                if (!(frame->hd.flags & NGHTTP2_FLAG_END_STREAM))
                         g_warn_if_reached ();
                 return 0;
         }
@@ -655,8 +690,11 @@ on_frame_recv_callback (nghttp2_session     *session,
                         data->metrics->response_body_bytes_received += frame->data.hd.length + FRAME_HEADER_SIZE;
                 if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM && data->body_istream) {
                         soup_body_input_stream_http2_complete (SOUP_BODY_INPUT_STREAM_HTTP2 (data->body_istream));
-                        if (data->state == STATE_READ_DATA_START)
+                        if (data->state == STATE_READ_DATA_START) {
                                 io_try_sniff_content (data, FALSE, data->item->cancellable);
+                                if (data->state == STATE_READ_DATA && data->item->async)
+                                        soup_http2_message_data_check_status (data);
+                        }
                 }
                 break;
         case NGHTTP2_RST_STREAM:
@@ -665,6 +703,12 @@ on_frame_recv_callback (nghttp2_session     *session,
                                                                        nghttp2_http2_strerror (frame->rst_stream.error_code)));
                 }
                 break;
+        case NGHTTP2_WINDOW_UPDATE:
+                h2_debug (io, data, "[RECV] WINDOW_UPDATE: increment=%d, total=%d", frame->window_update.window_size_increment,
+                          nghttp2_session_get_stream_remote_window_size (session, frame->hd.stream_id));
+                if (nghttp2_session_get_stream_remote_window_size (session, frame->hd.stream_id) > 0)
+                        io_try_write (io);
+                break;
         };
 
         return 0;
@@ -1279,7 +1323,8 @@ soup_client_message_io_http2_unpause (SoupClientMessageIO *iface,
 
         data->paused = FALSE;
 
-        soup_http2_message_data_check_status (data);
+        if (data->item->async)
+                soup_http2_message_data_check_status (data);
 }
 
 static void
@@ -1315,7 +1360,7 @@ soup_client_message_io_http2_is_open (SoupClientMessageIO *iface)
         if (!nghttp2_session_check_request_allowed (io->session))
                 return FALSE;
 
-        return !io->is_shutdown;
+        return !io->is_shutdown && !io->error;
 }
 
 static gboolean
@@ -1544,6 +1589,9 @@ soup_client_message_io_http2_destroy (SoupClientMessageIO *iface)
                 g_source_destroy (io->write_source);
                 g_source_unref (io->write_source);
         }
+
+        if (io->conn)
+                g_object_remove_weak_pointer (G_OBJECT (io->conn), (gpointer*)&io->conn);
         g_clear_object (&io->stream);
         g_clear_object (&io->close_task);
         g_clear_pointer (&io->session, nghttp2_session_del);
@@ -1626,15 +1674,18 @@ soup_client_message_io_http2_init (SoupClientMessageIOHTTP2 *io)
 #define MAX_HEADER_TABLE_SIZE 65536 /* Match size used by Chromium/Firefox */
 
 SoupClientMessageIO *
-soup_client_message_io_http2_new (GIOStream *stream, guint64 connection_id)
+soup_client_message_io_http2_new (SoupConnection *conn)
 {
         SoupClientMessageIOHTTP2 *io = g_new0 (SoupClientMessageIOHTTP2, 1);
         soup_client_message_io_http2_init (io);
 
-        io->stream = g_object_ref (stream);
+        io->conn = conn;
+        g_object_add_weak_pointer (G_OBJECT (io->conn), (gpointer*)&io->conn);
+
+        io->stream = g_object_ref (soup_connection_get_iostream (conn));
         io->istream = g_io_stream_get_input_stream (io->stream);
         io->ostream = g_io_stream_get_output_stream (io->stream);
-        io->connection_id = connection_id;
+        io->connection_id = soup_connection_get_id (conn);
 
         io->read_source = g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM (io->istream), NULL);
         g_source_set_name (io->read_source, "Soup HTTP/2 read source");
