@@ -101,11 +101,10 @@
  * If you want to handle the special "*" URI (eg, "OPTIONS *"), you
  * must explicitly register a handler for "*"; the default handler
  * will not be used for that case.
- * 
+ *
  * If you want to process https connections in addition to (or instead
- * of) http connections, you can either set the
- * SoupServer:tls-certificate property when creating the server, or
- * else call soup_server_set_ssl_cert_file() after creating it.
+ * of) http connections, you can set the #SoupServer:tls-certificate
+ * property.
  *
  * Once the server is set up, make one or more calls to
  * soup_server_listen(), soup_server_listen_local(), or
@@ -161,6 +160,8 @@ typedef struct {
 	GSList            *clients;
 
 	GTlsCertificate   *tls_cert;
+        GTlsDatabase      *tls_database;
+        GTlsAuthenticationMode tls_auth_mode;
 
 	char              *server_header;
 
@@ -184,6 +185,8 @@ enum {
 	PROP_0,
 
 	PROP_TLS_CERTIFICATE,
+        PROP_TLS_DATABASE,
+        PROP_TLS_AUTH_MODE,
 	PROP_RAW_PATHS,
 	PROP_SERVER_HEADER,
 
@@ -244,6 +247,7 @@ soup_server_finalize (GObject *object)
 	SoupServerPrivate *priv = soup_server_get_instance_private (server);
 
 	g_clear_object (&priv->tls_cert);
+        g_clear_object (&priv->tls_database);
 
 	g_free (priv->server_header);
 
@@ -268,10 +272,14 @@ soup_server_set_property (GObject *object, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_TLS_CERTIFICATE:
-		if (priv->tls_cert)
-			g_object_unref (priv->tls_cert);
-		priv->tls_cert = g_value_dup_object (value);
+                soup_server_set_tls_certificate (server, g_value_get_object (value));
 		break;
+        case PROP_TLS_DATABASE:
+                soup_server_set_tls_database (server, g_value_get_object (value));
+                break;
+        case PROP_TLS_AUTH_MODE:
+                soup_server_set_tls_auth_mode (server, g_value_get_enum (value));
+                break;
 	case PROP_RAW_PATHS:
 		priv->raw_paths = g_value_get_boolean (value);
 		break;
@@ -307,6 +315,12 @@ soup_server_get_property (GObject *object, guint prop_id,
 	case PROP_TLS_CERTIFICATE:
 		g_value_set_object (value, priv->tls_cert);
 		break;
+        case PROP_TLS_DATABASE:
+                g_value_set_object (value, priv->tls_database);
+                break;
+        case PROP_TLS_AUTH_MODE:
+                g_value_set_enum (value, priv->tls_auth_mode);
+                break;
 	case PROP_RAW_PATHS:
 		g_value_set_boolean (value, priv->raw_paths);
 		break;
@@ -434,10 +448,6 @@ soup_server_class_init (SoupServerClass *server_class)
 	 * A #GTlsCertificate that has a #GTlsCertificate:private-key
 	 * set. If this is set, then the server will be able to speak
 	 * https in addition to (or instead of) plain http.
-	 *
-	 * Alternatively, you can call soup_server_set_ssl_cert_file()
-	 * to have #SoupServer read in a a certificate from a file.
-	 *
 	 */
         properties[PROP_TLS_CERTIFICATE] =
 		g_param_spec_object ("tls-certificate",
@@ -445,9 +455,38 @@ soup_server_class_init (SoupServerClass *server_class)
 				     "GTlsCertificate to use for https",
 				     G_TYPE_TLS_CERTIFICATE,
 				     G_PARAM_READWRITE |
-                                     G_PARAM_CONSTRUCT_ONLY |
+                                     G_PARAM_CONSTRUCT |
                                      G_PARAM_STATIC_STRINGS);
 
+        /**
+         * SoupServer:tls-database:
+         *
+         * A #GTlsDatabase to use for validating SSL/TLS client certificates.
+         */
+        properties[PROP_TLS_DATABASE] =
+                g_param_spec_object ("tls-database",
+                                     "TLS database",
+                                     "GTlsDatabase to use for validating SSL/TLS client certificates",
+                                     G_TYPE_TLS_DATABASE,
+                                     G_PARAM_READWRITE |
+                                     G_PARAM_CONSTRUCT |
+                                     G_PARAM_STATIC_STRINGS);
+
+        /**
+         * SoupServer:tls-auth-mode:
+         *
+         * A #GTlsAuthenticationMode for SSL/TLS client authentication
+         */
+        properties[PROP_TLS_AUTH_MODE] =
+                g_param_spec_enum ("tls-auth-mode",
+                                   "TLS Authentication Mode",
+                                   "GTlsAuthenticationMode to use for SSL/TLS client authentication",
+                                   G_TYPE_TLS_AUTHENTICATION_MODE,
+                                   G_TLS_AUTHENTICATION_NONE,
+                                   G_PARAM_READWRITE |
+                                   G_PARAM_CONSTRUCT |
+                                   G_PARAM_STATIC_STRINGS);
+
         properties[PROP_RAW_PATHS] =
 		g_param_spec_boolean ("raw-paths",
 				      "Raw paths",
@@ -524,41 +563,131 @@ soup_server_new (const char *optname1, ...)
 }
 
 /**
- * soup_server_set_ssl_cert_file:
+ * soup_server_set_tls_certificate:
  * @server: a #SoupServer
- * @ssl_cert_file: path to a file containing a PEM-encoded SSL/TLS
- *   certificate.
- * @ssl_key_file: path to a file containing a PEM-encoded private key.
- * @error: return location for a #GError
- *
- * Sets @server up to do https, using the SSL/TLS certificate
- * specified by @ssl_cert_file and @ssl_key_file (which may point to
- * the same file).
- *
- * Alternatively, you can set the #SoupServer:tls-certificate property
- * at construction time, if you already have a #GTlsCertificate.
- *
- * Returns: success or failure.
+ * @certificate: a #GTlsCertificate
  *
+ * Sets @server up to do https, using the given SSL/TLS @certificate.
  */
-gboolean
-soup_server_set_ssl_cert_file  (SoupServer  *server,
-				const char  *ssl_cert_file,
-				const char  *ssl_key_file,
-				GError     **error)
+void
+soup_server_set_tls_certificate (SoupServer      *server,
+                                 GTlsCertificate *certificate)
 {
-	SoupServerPrivate *priv;
+        SoupServerPrivate *priv;
 
-	g_return_val_if_fail (SOUP_IS_SERVER (server), FALSE);
-	priv = soup_server_get_instance_private (server);
+        g_return_if_fail (SOUP_IS_SERVER (server));
 
-	if (priv->tls_cert)
-		g_object_unref (priv->tls_cert);
+        priv = soup_server_get_instance_private (server);
+        if (priv->tls_cert == certificate)
+                return;
 
-	priv->tls_cert = g_tls_certificate_new_from_files (ssl_cert_file,
-							   ssl_key_file,
-							   error);
-	return priv->tls_cert != NULL;
+        g_clear_object (&priv->tls_cert);
+        priv->tls_cert = certificate ? g_object_ref (certificate) : NULL;
+        g_object_notify_by_pspec (G_OBJECT (server), properties[PROP_TLS_CERTIFICATE]);
+}
+
+/**
+ * soup_server_get_tls_certificate:
+ * @server: a #SoupServer
+ *
+ * Gets the @server SSL/TLS certificate
+ *
+ * Returns: (transfer none) (nullable): a #GTlsCertificate or %NULL
+ */
+GTlsCertificate *
+soup_server_get_tls_certificate (SoupServer *server)
+{
+        SoupServerPrivate *priv;
+
+        g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
+
+        priv = soup_server_get_instance_private (server);
+        return priv->tls_cert;
+}
+
+/**
+ * soup_server_set_tls_database:
+ * @server: a #SoupServer
+ * @tls_database: a #GTlsDatabase
+ *
+ * Sets @server's #GTlsDatabase to use for validating SSL/TLS client certificates
+ */
+void
+soup_server_set_tls_database (SoupServer   *server,
+                              GTlsDatabase *tls_database)
+{
+        SoupServerPrivate *priv;
+
+        g_return_if_fail (SOUP_IS_SERVER (server));
+
+        priv = soup_server_get_instance_private (server);
+        if (priv->tls_database == tls_database)
+                return;
+
+        g_clear_object (&priv->tls_database);
+        priv->tls_database = tls_database ? g_object_ref (tls_database) : NULL;
+        g_object_notify_by_pspec (G_OBJECT (server), properties[PROP_TLS_DATABASE]);
+}
+
+/**
+ * soup_server_get_tls_database:
+ * @server: a #SoupServer
+ *
+ * Gets the @server SSL/TLS database
+ *
+ * Returns: (transfer none) (nullable): a #GTlsDatabase or %NULL
+ */
+GTlsDatabase *
+soup_server_get_tls_database (SoupServer *server)
+{
+        SoupServerPrivate *priv;
+
+        g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
+
+        priv = soup_server_get_instance_private (server);
+        return priv->tls_database;
+}
+
+/**
+ * soup_server_set_tls_auth_mode:
+ * @server: a #SoupServer
+ * @mode: a #GTlsAuthenticationMode
+ *
+ * Sets @server's #GTlsAuthenticationMode to use for SSL/TLS client authentication
+ */
+void
+soup_server_set_tls_auth_mode (SoupServer             *server,
+                               GTlsAuthenticationMode  mode)
+{
+        SoupServerPrivate *priv;
+
+        g_return_if_fail (SOUP_IS_SERVER (server));
+
+        priv = soup_server_get_instance_private (server);
+        if (priv->tls_auth_mode == mode)
+                return;
+
+        priv->tls_auth_mode = mode;
+        g_object_notify_by_pspec (G_OBJECT (server), properties[PROP_TLS_AUTH_MODE]);
+}
+
+/**
+ * soup_server_get_tls_auth_mode:
+ * @server: a #SoupServer
+ *
+ * Gets the @server SSL/TLS client authentication mode
+ *
+ * Returns: a #GTlsAuthenticationMode
+ */
+GTlsAuthenticationMode
+soup_server_get_tls_auth_mode (SoupServer *server)
+{
+        SoupServerPrivate *priv;
+
+        g_return_val_if_fail (SOUP_IS_SERVER (server), G_TLS_AUTHENTICATION_NONE);
+
+        priv = soup_server_get_instance_private (server);
+        return priv->tls_auth_mode;
 }
 
 /**
@@ -1073,9 +1202,15 @@ soup_server_listen_internal (SoupServer *server, SoupSocket *listener,
 			return FALSE;
 		}
 
-		g_object_set (G_OBJECT (listener),
-			      "tls-certificate", priv->tls_cert,
-			      NULL);
+                g_object_bind_property (server, "tls-certificate",
+                                        listener, "tls-certificate",
+                                        G_BINDING_SYNC_CREATE);
+                g_object_bind_property (server, "tls-database",
+                                        listener, "tls-database",
+                                        G_BINDING_SYNC_CREATE);
+                g_object_bind_property (server, "tls-auth-mode",
+                                        listener, "tls-auth-mode",
+                                        G_BINDING_SYNC_CREATE);
 	}
 
 	if (soup_socket_get_gsocket (listener) == NULL) {
@@ -1501,7 +1636,7 @@ get_or_create_handler (SoupServer *server, const char *exact_path)
  * @user_data: data for @callback
  * @destroy: destroy notifier to free @user_data
  *
- * Adds a handler to @server for requests under @path. If @path is
+ * Adds a handler to @server for requests prefixed by @path. If @path is
  * %NULL or "/", then this will be the default handler for all
  * requests that don't have a more specific handler. (Note though that
  * if you want to handle requests to the special "*" URI, you must
@@ -1566,7 +1701,7 @@ soup_server_add_handler (SoupServer            *server,
  * @user_data: data for @callback
  * @destroy: destroy notifier to free @user_data
  *
- * Adds an "early" handler to @server for requests under @path. Note
+ * Adds an "early" handler to @server for requests prefixed by @path. Note
  * that "normal" and "early" handlers are matched up together, so if
  * you add a normal handler for "/foo" and an early handler for
  * "/foo/bar", then a request to "/foo/bar" (or any path below it)
@@ -1644,7 +1779,7 @@ soup_server_add_early_handler (SoupServer            *server,
  * @user_data: data for @callback
  * @destroy: destroy notifier to free @user_data
  *
- * Adds a WebSocket handler to @server for requests under @path. (If
+ * Adds a WebSocket handler to @server for requests prefixed by @path. (If
  * @path is %NULL or "/", then this will be the default handler for
  * all requests that don't have a more specific handler.)
  *
