@@ -73,6 +73,8 @@ enum {
 	PROP_SSL,
 	PROP_TLS_CERTIFICATE,
 	PROP_TLS_CERTIFICATE_ERRORS,
+        PROP_TLS_PROTOCOL_VERSION,
+        PROP_TLS_CIPHERSUITE_NAME,
         PROP_FORCE_HTTP1,
 
 	LAST_PROPERTY
@@ -194,6 +196,12 @@ soup_connection_get_property (GObject *object, guint prop_id,
 	case PROP_TLS_CERTIFICATE_ERRORS:
 		g_value_set_flags (value, soup_connection_get_tls_certificate_errors (SOUP_CONNECTION (object)));
 		break;
+        case PROP_TLS_PROTOCOL_VERSION:
+                g_value_set_enum (value, soup_connection_get_tls_protocol_version (SOUP_CONNECTION (object)));
+                break;
+        case PROP_TLS_CIPHERSUITE_NAME:
+                g_value_set_string (value, soup_connection_get_tls_ciphersuite_name (SOUP_CONNECTION (object)));
+                break;
 	case PROP_FORCE_HTTP1:
 		g_value_set_boolean (value, priv->force_http1);
 		break;
@@ -321,6 +329,21 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
                                     G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
                                     G_PARAM_READABLE |
                                     G_PARAM_STATIC_STRINGS);
+        properties[PROP_TLS_PROTOCOL_VERSION] =
+                g_param_spec_enum ("tls-protocol-version",
+                                   "TLS Protocol Version",
+                                   "TLS protocol version negotiated for this connection",
+                                   G_TYPE_TLS_PROTOCOL_VERSION,
+                                   G_TLS_PROTOCOL_VERSION_UNKNOWN,
+                                   G_PARAM_READABLE |
+                                   G_PARAM_STATIC_STRINGS);
+        properties[PROP_TLS_CIPHERSUITE_NAME] =
+                g_param_spec_string ("tls-ciphersuite-name",
+                                     "TLS Ciphersuite Name",
+                                     "Name of TLS ciphersuite negotiated for this connection",
+                                     NULL,
+                                     G_PARAM_READABLE |
+                                     G_PARAM_STATIC_STRINGS);
         properties[PROP_FORCE_HTTP1] =
                 g_param_spec_boolean ("force-http1",
                                       "Force HTTP 1.x",
@@ -455,10 +478,10 @@ soup_connection_create_io_data (SoupConnection *conn)
         switch (priv->http_version) {
         case SOUP_HTTP_1_0:
         case SOUP_HTTP_1_1:
-                priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+                priv->io_data = soup_client_message_io_http1_new (conn);
                 break;
         case SOUP_HTTP_2_0:
-                priv->io_data = soup_client_message_io_http2_new (priv->iostream, priv->id);
+                priv->io_data = soup_client_message_io_http2_new (conn);
                 break;
         }
 }
@@ -522,6 +545,18 @@ tls_connection_peer_certificate_changed (SoupConnection *conn)
 	g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_TLS_CERTIFICATE]);
 }
 
+static void
+tls_connection_protocol_version_changed (SoupConnection *conn)
+{
+        g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_TLS_PROTOCOL_VERSION]);
+}
+
+static void
+tls_connection_ciphersuite_name_changed (SoupConnection *conn)
+{
+        g_object_notify_by_pspec (G_OBJECT (conn), properties[PROP_TLS_CIPHERSUITE_NAME]);
+}
+
 static GTlsClientConnection *
 new_tls_connection (SoupConnection    *conn,
                     GSocketConnection *connection,
@@ -564,6 +599,12 @@ new_tls_connection (SoupConnection    *conn,
 	g_signal_connect_object (tls_connection, "notify::peer-certificate",
 				 G_CALLBACK (tls_connection_peer_certificate_changed),
 				 conn, G_CONNECT_SWAPPED);
+        g_signal_connect_object (tls_connection, "notify::protocol-version",
+				 G_CALLBACK (tls_connection_protocol_version_changed),
+				 conn, G_CONNECT_SWAPPED);
+        g_signal_connect_object (tls_connection, "notify::ciphersuite-name",
+				 G_CALLBACK (tls_connection_ciphersuite_name_changed),
+				 conn, G_CONNECT_SWAPPED);
 
         return tls_connection;
 }
@@ -820,7 +861,7 @@ tunnel_handshake_ready_cb (GTlsConnection *tls_connection,
                 soup_connection_event (conn, G_SOCKET_CLIENT_COMPLETE, NULL);
 
                 g_assert (!priv->io_data);
-                priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+                priv->io_data = soup_client_message_io_http1_new (conn);
 
                 g_task_return_boolean (task, TRUE);
         } else {
@@ -911,7 +952,7 @@ soup_connection_tunnel_handshake (SoupConnection *conn,
         soup_connection_event (conn, G_SOCKET_CLIENT_COMPLETE, NULL);
 
         g_assert (!priv->io_data);
-        priv->io_data = soup_client_message_io_http1_new (priv->iostream);
+        priv->io_data = soup_client_message_io_http1_new (conn);
 
         return TRUE;
 }
@@ -1110,10 +1151,8 @@ soup_connection_setup_message_io (SoupConnection *conn,
         if (priv->proxy_uri && soup_message_get_method (msg) == SOUP_METHOD_CONNECT)
                 set_proxy_msg (conn, msg);
 
-        if (!soup_client_message_io_is_reusable (priv->io_data)) {
-                g_clear_pointer (&priv->io_data, soup_client_message_io_destroy);
-                soup_connection_create_io_data (conn);
-        }
+        if (!soup_client_message_io_is_reusable (priv->io_data))
+                g_warn_if_reached ();
 
         return priv->io_data;
 }
@@ -1148,13 +1187,35 @@ soup_connection_get_tls_certificate_errors (SoupConnection *conn)
 	return g_tls_connection_get_peer_certificate_errors (G_TLS_CONNECTION (priv->connection));
 }
 
+GTlsProtocolVersion
+soup_connection_get_tls_protocol_version (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        if (!G_IS_TLS_CONNECTION (priv->connection))
+                return G_TLS_PROTOCOL_VERSION_UNKNOWN;
+
+        return g_tls_connection_get_protocol_version (G_TLS_CONNECTION (priv->connection));
+}
+
+char *
+soup_connection_get_tls_ciphersuite_name (SoupConnection *conn)
+{
+        SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
+
+        if (!G_IS_TLS_CONNECTION (priv->connection))
+                return NULL;
+
+        return g_tls_connection_get_ciphersuite_name (G_TLS_CONNECTION (priv->connection));
+}
+
 void
 soup_connection_set_tls_client_certificate (SoupConnection  *conn,
                                             GTlsCertificate *certificate)
 {
         SoupConnectionPrivate *priv = soup_connection_get_instance_private (conn);
 
-        if (G_IS_TLS_CONNECTION (priv->connection)) {
+        if (G_IS_TLS_CONNECTION (priv->connection) && certificate) {
                 g_tls_connection_set_certificate (G_TLS_CONNECTION (priv->connection),
                                                   certificate);
                 g_clear_object (&priv->tls_client_cert);
@@ -1165,7 +1226,7 @@ soup_connection_set_tls_client_certificate (SoupConnection  *conn,
                 return;
 
         g_clear_object (&priv->tls_client_cert);
-        priv->tls_client_cert = g_object_ref (certificate);
+        priv->tls_client_cert = certificate ? g_object_ref (certificate) : NULL;
 }
 
 void
