@@ -277,3 +277,19 @@ soup_get_accept_languages_from_system (void)
 
 	return langs_str;
 }
+
+const char *
+soup_http_version_to_string (SoupHTTPVersion version)
+{
+        switch (version) {
+        case SOUP_HTTP_1_0:
+                return "1.0";
+        case SOUP_HTTP_1_1:
+                return "1.1";
+        case SOUP_HTTP_2_0:
+                return "2";
+        }
+
+        g_assert_not_reached ();
+        return NULL;
+}
