@@ -255,13 +255,32 @@ reuse_test_authenticate (SoupMessage *msg,
 	return TRUE;
 }
 
+static void
+reuse_preconnect_finished (SoupSession   *session,
+                           GAsyncResult  *result,
+                           GError       **error)
+{
+        g_assert_false (soup_session_preconnect_finish (session, result, error));
+}
+
+static void
+reuse_websocket_connect_finished (SoupSession   *session,
+                                  GAsyncResult  *result,
+                                  GError       **error)
+{
+        g_assert_false (soup_session_websocket_connect_finish (session, result, error));
+}
+
 static void
 do_msg_reuse_test (void)
 {
 	SoupSession *session;
 	SoupMessage *msg;
+        GBytes *body;
 	GUri *uri;
 	guint *signal_ids, n_signal_ids;
+        GInputStream *stream;
+        GError *error = NULL;
 
 	g_test_bug ("559054");
 
@@ -273,26 +292,69 @@ do_msg_reuse_test (void)
 	msg = soup_message_new_from_uri ("GET", base_uri);
 	g_signal_connect (msg, "authenticate",
                           G_CALLBACK (reuse_test_authenticate), NULL);
-	soup_test_session_async_send (session, msg, NULL, NULL);
+	body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert_nonnull (body);
 	ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
+        g_bytes_unref (body);
 
 	debug_printf (1, "  Redirect message\n");
 	uri = g_uri_parse_relative (base_uri, "/redirect", SOUP_HTTP_URI_FLAGS, NULL);
 	soup_message_set_uri (msg, uri);
 	g_uri_unref (uri);
-	soup_test_session_async_send (session, msg, NULL, NULL);
+	body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert_nonnull (body);
 	g_assert_true (soup_uri_equal (soup_message_get_uri (msg), base_uri));
 	ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
+        g_bytes_unref (body);
 
 	debug_printf (1, "  Auth message\n");
 	uri = g_uri_parse_relative (base_uri, "/auth", SOUP_HTTP_URI_FLAGS, NULL);
 	soup_message_set_uri (msg, uri);
 	g_uri_unref (uri);
-	soup_test_session_async_send (session, msg, NULL, NULL);
+	body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert_nonnull (body);
 	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+        g_bytes_unref (body);
 	soup_message_set_uri (msg, base_uri);
-	soup_test_session_async_send (session, msg, NULL, NULL);
+	body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert_nonnull (body);
 	ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
+        g_bytes_unref (body);
+
+        debug_printf (1, "  Reuse before finishing\n");
+        msg = soup_message_new_from_uri ("GET", base_uri);
+        stream = soup_test_request_send (session, msg, NULL, 0, &error);
+        g_assert_no_error (error);
+        g_assert_null (soup_test_request_send (session, msg, NULL, 0, &error));
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        g_assert_null (soup_test_session_async_send (session, msg, NULL, &error));
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        g_assert_null (soup_session_send (session, msg, NULL, &error));
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        g_assert_null (soup_session_send_and_read (session, msg, NULL, &error));
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        soup_session_preconnect_async (session, msg, G_PRIORITY_DEFAULT, NULL,
+                                       (GAsyncReadyCallback)reuse_preconnect_finished, &error);
+        while (error == NULL)
+                g_main_context_iteration (NULL, TRUE);
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        soup_session_websocket_connect_async (session, msg, NULL, NULL, G_PRIORITY_DEFAULT, NULL,
+                                              (GAsyncReadyCallback)reuse_websocket_connect_finished, &error);
+        while (error == NULL)
+                g_main_context_iteration (NULL, TRUE);
+        g_assert_error (error, SOUP_SESSION_ERROR, SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
+        g_clear_error (&error);
+        g_object_unref (stream);
+
+        while (g_main_context_pending (NULL))
+                g_main_context_iteration (NULL, FALSE);
+
+        ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
 
 	soup_test_session_abort_unref (session);
 	g_object_unref (msg);
@@ -760,6 +822,45 @@ do_remote_address_test (void)
         soup_test_session_abort_unref (session);
 }
 
+static void
+redirect_handler (SoupMessage *msg,
+                  SoupSession *session)
+{
+        SoupMessage *new_msg;
+        GBytes *body;
+
+        new_msg = soup_message_new_from_uri ("GET", base_uri);
+        body = soup_test_session_async_send (session, new_msg, NULL, NULL);
+        g_assert_nonnull (body);
+        g_assert_cmpstr (g_bytes_get_data (body, NULL), ==, "index");
+        g_object_unref (new_msg);
+}
+
+static void
+do_new_request_on_redirect_test (void)
+{
+        SoupSession *session;
+        GUri *uri;
+        SoupMessage *msg;
+        GBytes *body;
+
+        session = soup_test_session_new (NULL);
+
+        uri = g_uri_parse_relative (base_uri, "/redirect", SOUP_HTTP_URI_FLAGS, NULL);
+        msg = soup_message_new_from_uri ("GET", uri);
+        g_signal_connect_after (msg, "got-body",
+                                G_CALLBACK (redirect_handler),
+                                session);
+        body = soup_test_session_async_send (session, msg, NULL, NULL);
+        g_assert_nonnull (body);
+        g_assert_cmpstr (g_bytes_get_data (body, NULL), ==, "index");
+
+        g_bytes_unref (body);
+        g_object_unref (msg);
+        g_uri_unref (uri);
+        soup_test_session_abort_unref (session);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -794,6 +895,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/misc/msg-flags", do_msg_flags_test);
         g_test_add_func ("/misc/connection-id", do_connection_id_test);
         g_test_add_func ("/misc/remote-address", do_remote_address_test);
+        g_test_add_func ("/misc/new-request-on-redirect", do_new_request_on_redirect_test);
 
 	ret = g_test_run ();
 
